/*                                                                    */
/* wmab-0.3                                                           */
/*                                                                    */
/* A  WindowMaker dockable  application that  allows laptop  users to */
/* graphically  monitor  the  status  of their  power  source.  (I.e. */
/* whether or not AC or battery is in use as well as how long it will */
/* take  to  drain  or  charge the  battery.)  It  supports  multiple */
/* batteries simple with -b option.                                   */
/*                                                                    */
/* Written by Filippo Panessa  <kalem@linux.it> and copyrighted under */
/* GPL.                                                               */
/*                                                                    */
/* Based  on wmapm-3.1.0  originally written  (and copyrighted  under */
/* GPL)  by  Chris  D.  Faulhaber  <jedgar@fxp.org>  and  extensively */
/* modified by Michael G. Henderson <mghenderson@lanl.gov>.           */
/*                                                                    */
/*                                                                    */
/* This  program is  free software;  you can  redistribute it  and/or */
/* modify it  under the terms  of the  GNU General Public  License as */
/* published by  the Free Software  Foundation; either version  2, or */
/* (at your option) any later version.                                */
/*                                                                    */
/* This program  is distributed in the  hope that it will  be useful, */
/* but WITHOUT  ANY WARRANTY;  without even  the implied  warranty of */
/* MERCHANTABILITY or FITNESS  FOR A PARTICULAR PURPOSE.  See the GNU */
/* General Public License for more details.                           */
/*                                                                    */
/* You should have received a copy  of the GNU General Public License */
/* along with this  program (see the file COPYING); if  not, write to */
/* the Free Software  Foundation, Inc., 59 Temple Place  - Suite 330, */
/* Boston, MA 02111-1307, USA                                         */
/*                                                                    */
/*                                                                    */
/*                                                                    */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <X11/X.h>
#include <X11/xpm.h>

#include "wmab.h"
#include "libacpi.h"
#include "../wmgeneral/wmgeneral.h"

#include "../pixmaps/wmab_master.xpm"
#include "../pixmaps/wmab_master_LowColor.xpm"
#include "../pixmaps/wmab_mask.xbm"

void ParseCMDLine (int argc, char *argv[]);
void pressEvent (XButtonEvent * xev);

int Battery = 1;		/* Use Battery 1 for default */
int CriticalLevel = 10;
int LowLevel = 40;
float BlinkRate = 3.0;		/* blinks per second */
float UpdateRate = 0.8;		/* Number of updates per second */
int Beep = 0;			/* Controls beeping when you 
				   get to CriticalLevel: 
				   Off by default */
int Volume = 50;		/* ring bell at 50% volume */
int UseLowColorPixmap = 0;	/* Use a lower number of colors 
				   for the poor saps on 
				   8-bit displays -- common
				   on laptops! */

/*  
 *   main  
 */

int
main (int argc, char *argv[])
{

  int rtime, hour, min, digit;
  XEvent event;
  int m, n, k, Toggle;
  long int r, rMax, s, sMax;

  int error;

  BlinkRate = 3.0;
  UpdateRate = 1.0 / 1.25;

  /*
   *  Parse any command line arguments.
   */
  ParseCMDLine (argc, argv);

  BlinkRate = (BlinkRate >= 0.0) ? BlinkRate : -1.0 * BlinkRate;
  UpdateRate = (UpdateRate >= 0.0) ? UpdateRate : -1.0 * UpdateRate;

  /*  
   *  Check for ACPI support 
   */

  error = check_acpi ();

  if (error == 1)
    {
      fprintf (stderr, "This system does not support ACPI\n");
      exit (1);
    }
  if (error == 2)
    {
      fprintf (stderr, "No batteries or ACPI not supported\n");
      exit (1);
    }

  if (UseLowColorPixmap)
    openXwindow (argc, argv, wmab_master_LowColor, wmab_mask_bits,
		 wmab_mask_width, wmab_mask_height);
  else
    openXwindow (argc, argv, wmab_master, wmab_mask_bits, wmab_mask_width,
		 wmab_mask_height);

  copyXPMArea ((Battery) * 6 + 4, 69, 6, 7, 23, 50);

  acpistate = calloc (1, sizeof (ACPIstate));
  acpiinfo = calloc (1, sizeof (ACPIinfo));
  acadstate = calloc (1, sizeof (ACADstate));

  /*
   *     Show battery number
   */

  copyXPMArea ((Battery) * 6 + 4, 69, 6, 7, 23, 50);

  /*
   *     Loop until we die...
   */

  while (1)
    {
      /* 
       *   Check AC status.
       */


      read_acad_state ();

      if (acadstate->state == 1)
	/*
	 *   AC on-line. I.e. we are "plugged-in".
	 */
	copyXPMArea (68, 6, 12, 7, 31, 35);
      else
	/*
	   *   AC off-line. I.e. we are using battery.
	 */
	copyXPMArea (68, 20, 12, 7, 31, 35);

      /* 
       *    Paste up the default charge status and time
       */
      copyXPMArea (104, 6, 5, 7, 6, 7);
      copyXPMArea (83, 93, 41, 9, 15, 7);

      /*
       *  Check to see if we are charging.
       */

      read_acpi_info (Battery - 1);
      read_acpi_state (Battery - 1);

      if (acpistate->state == CHARGING)
	{

	  /*
	   *   Battery Status: Charging.
	   */
	  copyXPMArea (82, 68, 7, 9, 6, 7);

	}
      else
	{
	  copyXPMArea (88, 68, 7, 9, 6, 7);
	}

      /* 
       *  Paste up the "Time Left". This time means:
       *  
       *         If not charging: Time left before battery drains to 0% 
       *         If charging:     Time left before battery gets to maximum
       *
       */

      if ((acpistate->rtime >= 86400) || (acpistate->rtime < -1))
	{

	  /*
	   *  If battery_time is too large, it likely means that there is
	   *  no charging or discharging going on. So just display a "null"
	   *  indicator (--:--). 
	   *
	   */
	  copyXPMArea (83, 106, 41, 9, 15, 7);

	}
      else if (acpistate->rtime >= 0)
	{

	  rtime = acpistate->rtime;

	  if (acpistate->percentage == 100)
	    rtime = 0;

	  hour = rtime / 60;

	  min = rtime % 60;

	  copyXPMArea ((hour / 10) * 7 + 5, 93, 7, 9, 21, 7);
	  /* Show 10's (hour) */

	  copyXPMArea ((hour % 10) * 7 + 5, 93, 7, 9, 29, 7);
	  /* Show 1's (hour)  */

	  copyXPMArea (76, 93, 2, 9, 38, 7);
	  /* colon            */

	  copyXPMArea ((min / 10) * 7 + 5, 93, 7, 9, 42, 7);
	  /* Show 10's (min)  */

	  copyXPMArea ((min % 10) * 7 + 5, 93, 7, 9, 50, 7);
	  /* Show 1's (min)   */

	}

      /* 
       *   Do Battery Percentage.
       */
      copyXPMArea (76, 81, 19, 7, 7, 34);	/* Show Default % */
      copyXPMArea (66, 31, 49, 9, 7, 21);	/* Show Default Meter */

      if (acpistate->percentage == 100)
	{
	  copyXPMArea (15, 81, 1, 7, 7, 34);
	  /* If 100%, show 100% */

	  copyXPMArea (5, 81, 6, 7, 9, 34);
	  copyXPMArea (5, 81, 6, 7, 15, 34);
	  copyXPMArea (64, 81, 7, 7, 21, 34);	/* Show '%' */
	  copyXPMArea (66, 42, 49, 9, 7, 21);	/* Show Meter */
	}
      else
	{

	  if (acpistate->percentage >= 10)
	    copyXPMArea ((acpistate->percentage / 10) * 6 + 4, 81, 6,
			 7, 9, 34);
	  /* Show 10's */
	  copyXPMArea ((acpistate->percentage % 10) * 6 + 4, 81, 6, 7, 15,
		       34);
	  /* Show 1's */
	  copyXPMArea (64, 81, 7, 7, 21, 34);
	  /* Show '%' */

	  /*
	   *  Show Meter
	   */
	  k = acpistate->percentage * 49 / 100;
	  copyXPMArea (66, 42, k, 9, 7, 21);
	  if (k % 2)
	    copyXPMArea (66 + k - 1, 52, 1, 9, 7 + k - 1, 21);
	  else
	    copyXPMArea (66 + k, 52, 1, 9, 7 + k, 21);
	}

      /*
       *  This controls battery LED
       */

      if (acpistate->present == 1)
	{

	  if (acpistate->rcapacity <=
	      acpiinfo->design_capacity_warning
	      || acpistate->percentage <= CriticalLevel)
	    {

	      /*
	       *  Battery Status: Critical.
	       *  Blink the red led on/off...
	       */
	      if (Toggle || (BlinkRate == 0.0))
		{
		  if (Beep)
		    XBell (display, Volume);
		  Toggle = 0;
		  copyXPMArea (4, 105, 16, 10, 43, 34);
		}
	      else
		{
		  Toggle = 1;
		  copyXPMArea (58, 105, 16, 10, 43, 34);
		}

	    }
	  else if (acpistate->rcapacity ==
		   acpiinfo->design_capacity_low
		   || acpistate->percentage <= LowLevel)

	    {

	      /*
	       *  Battery Status: Low.
	       */
	      copyXPMArea (22, 105, 16, 10, 43, 34);	/* turn ON  yellow */

	    }
	  else if (acpistate->state == POWER
		   || acpistate->percentage > LowLevel)
	    {

	      /*
	       *  Battery Status: High.
	       */
	      copyXPMArea (40, 105, 16, 10, 43, 34);	/* turn ON  green  */
	    }

	}
      else
	{
	  copyXPMArea (95, 19, 16, 10, 43, 34);
	}

      /* 
       * Process any pending X events.
       */

      while (XPending (display))
	{
	  XNextEvent (display, &event);
	  switch (event.type)
	    {
	    case Expose:
	      RedrawWindow ();
	      break;
	    case ButtonPress:
	      pressEvent (&event.xbutton);
	      break;
	    case ButtonRelease:
	      break;
	    }
	}

      /* 
       *  Redraw and wait for next update 
       */
      RedrawWindow ();
      usleep (DELAY);
    }
}

void
pressEvent (XButtonEvent * xev)
{

  int x = xev->x;
  int y = xev->y;

  if (x >= 32 && y >= 47 && x <= 45 && y <= 59)
    {
      copyXPMArea (98, 76, 14, 13, 32, 47);
      copyXPMArea (132, 75, 56, 41, 4, 4);

      if (Battery == 1)
	Battery = batt_count;
      else
	Battery--;

      // Show battery number

      copyXPMArea ((Battery) * 6 + 4, 69, 6, 7, 23, 50);

      RedrawWindow ();

      copyXPMArea (98, 62, 14, 13, 32, 47);
    }

  if (x >= 46 && y >= 47 && x <= 59 && y <= 59)
    {
      copyXPMArea (112, 76, 14, 13, 46, 47);
      copyXPMArea (132, 75, 56, 41, 4, 4);

      if (Battery == batt_count)
	Battery = 1;
      else
	Battery++;

      // Show battery number
      copyXPMArea ((Battery) * 6 + 4, 69, 6, 7, 23, 50);

      RedrawWindow ();

      copyXPMArea (112, 62, 14, 13, 46, 47);
    }

  return;

}

/* 
 *   ParseCMDLine()  
 */
void
ParseCMDLine (int argc, char *argv[])
{
  char *cmdline;
  int i;

  for (i = 1; i < argc; i++)
    {
      cmdline = argv[i];

      if (cmdline[0] == '-')
	{
	  switch (cmdline[1])
	    {
	    case 'd':
	      ++i;
	      break;
	    case 'C':
	      CriticalLevel = atoi (argv[++i]);
	      break;
	    case 'L':
	      LowLevel = atoi (argv[++i]);
	      break;
	    case 'l':
	      UseLowColorPixmap = 1;
	      break;
	    case 'B':
	      Beep = 1;
              Volume = atoi (argv[++i]);
	      break;
	    default:
	      printf ("\nwmab version: %s\n", WMAB_VERSION);
	      printf ("usage:\n");
	      printf ("\t-display <display>\tUse alternate display\n");
	      printf ("\t-l\t\t\tUse a low-color pixmap\n");
	      printf
		("\t-L <LowLevel>\t\tDefine level at which yellow LED turns on\n");
	      printf
		("\t-C <CriticalLevel>\tDefine level at which red LED turns on\n");
	      printf
		("\t-B <Volume>\t\tBeep at Critical Level (-100%% to 100%%)\n");
	      printf ("\t-h\t\t\tDisplay help screen\n\n");
	      exit (1);
	    }
	}
    }
}
